(function() {
	var ormmaview = window.ormmaview = {};
	var expandProperties = {
		useBackground: false,
		backgroundColor: "#ffffff",
		backgroundOpacity: 0,
		lockOrientation: false,
		useCustomClose: false,
		width: 0,
		height: 0,
		isModal: true
	};
	
	var listeners = {};
	
	ormmaview.fireAssetReadyEvent = function(alias, URL) {
		var handlers = listeners["assetReady"];
		if (handlers != null) {
			for (var i = 0; i < handlers.length; i++) {
				handlers[i](alias, URL);
			}
		}
		/*return "OK";*/
	};
	ormmaview.fireAssetRemovedEvent = function(alias) {
		var handlers = listeners["assetRemoved"];
		if (handlers != null) {
			for (var i = 0; i < handlers.length; i++) {
				handlers[i](alias)
			}
		}
		/*return "OK";*/
	};
	ormmaview.fireAssetRetiredEvent = function(alias) {
		var handlers = listeners["assetRetired"];
		if (handlers != null) {
			for (var i = 0; i < handlers.length; i++) {
				handlers[i](alias);
			}
		}
		/*return "OK";*/
	};
	ormmaview.fireChangeEvent = function(properties) {
		var handlers = listeners["change"];
		if (handlers != null) {
			for (var i = 0; i < handlers.length; i++) {
				handlers[i](properties);
			}
		}
		/*return "OK";*/
	};
	ormmaview.fireErrorEvent = function(message, action) {
		var handlers = listeners["error"];
		if (handlers != null) {
			for (var i = 0; i < handlers.length; i++) {
				handlers[i](message, action);
			}
		}
		ormmaview.onError(message, action);
		/*return "OK"*/
	};
	ormmaview.fireShakeEvent = function() {
		var handlers = listeners["shake"];
		if (handlers != null) {
			for (var i = 0; i < handlers.length; i++) {
				handlers[i]();
			}
		}
		/*return "OK";*/
	};
	ormmaview.fireResponseEvent = function(uri, response) {
		var data = {
			uri: uri,
			response: decode64(response)
		};
		var handlers = listeners["response"];
		if (handlers != null) {
			for (var i = 0; i < handlers.length; i++) {
				handlers[i](data);
			}
		}
		/*return "OK";*/
	};	
	ormmaview.zeroPad = function(number) {
		var text = "";
		if (number < 10) {
			text += "0";
		}
		text += number;
		return text;
	};
	ormmaview.addEventListener = function(event, listener) {
		var handlers = listeners[event];
		if (handlers == null) {
			listeners[event] = [];
			handlers = listeners[event];
		}
		for (var handler in handlers) {
			if (listener == handler) {
				return;
			}
		}
		handlers.push(listener)
	};	
	ormmaview.onError = function(message, action) {
		ORMMAUtilityControllerBridge.onError(message, action)
	};
	ormmaview.showAlert = function(message) {
		ORMMAUtilityControllerBridge.showAlert(message)
	};	
	ormmaview.startLocationListener = function() {
		ORMMALocationControllerBridge.startLocationListener();
	};
	ormmaview.stopLocationListener = function() {
		ORMMALocationControllerBridge.stopLocationListener();
	};
	ormmaview.startOrientationListener = function() {
		ORMMADisplayControllerBridge.startConfigurationListener();
	};
	ormmaview.stopOrientationListener = function() {
		ORMMADisplayControllerBridge.stopConfigurationListener();
	};
	ormmaview.activate = function(event) {
		ORMMAUtilityControllerBridge.activate(event);
	};
	ormmaview.deactivate = function(event) {
		ORMMAUtilityControllerBridge.deactivate(event);
	};
	ormmaview.removeEventListener = function(event, listener) {
		var handlers = listeners[event];
		if (handlers != null) {
			handlers.remove(listener);
		}
	};
	ormmaview.close = function() {
		try {
			ORMMADisplayControllerBridge.close();
		} catch (e) {
			ormmaview.showAlert("close: " + e);
		}
	};
	ormmaview.expand = function(dimensions, URL) {
		console.log("Expand");
		try {
			this.dimensions = dimensions;
			ORMMADisplayControllerBridge.expand(
					ormmaview.stringify(dimensions), URL, 
					ormmaview.stringify(expandProperties));
		} catch (e) {
			ormmaview.showAlert("executeNativeExpand: " + e + ", dimensions = " 
					+ dimensions + ", URL = " + URL + 
					", expandProperties = " + expandProperties);
		}
	};
	ormmaview.hide = function() {
		try {
			ORMMADisplayControllerBridge.hide();
		} catch (e) {
			ormmaview.showAlert("hide: " + e);
		}
	};
	ormmaview.open = function(URL, controls) {
		var back = false;
		var forward = false;
		var refresh = false;
		if (controls == null) {
			back = true;
			forward = true;
			refresh = true;
		} else {
			for (var i = 0; i < controls.length; i++) {
				if (controls[i] == "none" && i > 0) {
					self.fireErrorEvent("none must be the only navigation element present.", "open");
					return;
				} else if (controls[i] == "all") {
					if (i > 0) {
						self.fireErrorEvent("none must be the only navigation element present.", "open");
						return;
					}
					back = true;
					forward = true;
					refresh = true;
				} else if (controls[i] == "back") {
					back = true;
				} else if (controls[i] == "forward") {
					forward = true;
				} else if (controls[i] == "refresh") {
					refresh = true;
				}
			}
		}
		try {
			ORMMADisplayControllerBridge.open(URL, back, forward, refresh)
		} catch (e) {
			ormmaview.showAlert("open: " + e)
		}
	};
	ormmaview.openMap = function(POI, fullscreen) {
		try {
			ORMMADisplayControllerBridge.openMap(POI, fullscreen);
		} catch (e) {
			ormmaview.showAlert("openMap: " + e);
		}
	};
	ormmaview.playAudio = function(URL, properties) {
		var autoPlay = false,
			controls = false,
			loop = false,
			position = false,
			startStyle = "normal",
			stopStyle = "normal";
		if (properties != null) {
			if (typeof properties.autoplay != "undefined" && properties.autoplay != null) {
				autoPlay = true
			}
			if (typeof properties.controls != "undefined" && properties.controls != null) {
				controls = true
			}
			if (typeof properties.loop != "undefined" && properties.loop != null) {
				loop = true
			}
			if (typeof properties.position != "undefined" && properties.position != null) {
				position = true
			}
			if (typeof properties.startStyle != "undefined" && properties.startStyle != null) {
				startStyle = properties.startStyle
			}
			if (typeof properties.stopStyle != "undefined" && properties.stopStyle != null) {
				stopStyle = properties.stopStyle
			}
			if (position) {
				autoPlay = true;
				controls = false;
				loop = false;
				stopStyle = "exit";
			}
			if (loop) {
				stopStyle = "normal";
				controls = true;
			}
			if (!autoPlay) {
				controls = true;
			}
			if (!controls) {
				stopStyle = "exit";
			}
		}
		try {
			ORMMADisplayControllerBridge.playAudio(URL, autoPlay, controls, loop, position, startStyle, stopStyle)
		} catch (e) {
			
		}
	};
	ormmaview.playVideo = function(URL, properties) {
		var audioMuted = false,
				autoPlay = false,
				controls = false,
				loop = false,
				position = [-1, -1, -1, -1],
				startStyle = "normal",
				stopStyle = "normal";
		if (properties != null) {
			if (typeof properties.audio != "undefined" && properties.audio != null) {
				audioMuted = true
			}
			if (typeof properties.autoplay != "undefined" && properties.autoplay != null) {
				autoPlay = true
			}
			if (typeof properties.controls != "undefined" && properties.controls != null) {
				controls = true
			}
			if (typeof properties.loop != "undefined" && properties.loop != null) {
				loop = true
			}
			if (typeof properties.position != "undefined" && properties.position != null) {
				inline = new Array(4);
				inline[0] = properties.position.top;
				inline[1] = properties.position.left;
				if (typeof properties.width != "undefined" && properties.width != null) {
					inline[2] = properties.width;
				} else {
				}
				if (typeof properties.height != "undefined" && properties.height != null) {
					inline[3] = properties.height;
				} else {
				}
			}
			if (typeof properties.startStyle != "undefined" && properties.startStyle != null) {
				startStyle = properties.startStyle;
			}
			if (typeof properties.stopStyle != "undefined" && properties.stopStyle != null) {
				stopStyle = properties.stopStyle;
			}
			if (loop) {
				stopStyle = "normal";
				controls = true;
			}
			if (!autoPlay)
				controls = true;
			if (!controls) {
				stopStyle = "exit";
			}
			if (position[0] == -1 || position[1] == -1) {
				startStyle = "fullscreen";
			}
		}
		try {
			ORMMADisplayControllerBridge.playVideo(URL, audioMuted, autoPlay, controls, loop, position, startStyle, stopStyle)
		} catch (e) {
		}
	};
	ormmaview.resize = function(width, height) {
		try {
			ORMMADisplayControllerBridge.resize(width, height)
		} catch (e) {
		}
	};
	ormmaview.getExpandProperties = function() {
		return expandProperties;
	};
	ormmaview.setExpandProperties = function(properties) {
		expandProperties = properties;
	};
	ormmaview.useCustomClose = function(flag) {
		expandProperties.useCustomClose = flag;
		ORMMADisplayControllerBridge.useCustomClose(flag);
	};
	ormmaview.storePicture = function(URL) {
		ORMMADisplayControllerBridge.storePicture(URL);
	};
	ormmaview.show = function() {
		try {
			ORMMADisplayControllerBridge.show();
		} catch (e) {
			ormmaview.showAlert("show: " + e);
		}
	};
	ormmaview.createEvent = function(date, title, body) {
		var msecs = (date.getTime() - date.getMilliseconds());
		try {
			ORMMAUtilityControllerBridge.createEvent(msecs.toString(), title, body);
		} catch (ex) {
			ormmaview.showAlert("createEvent: " + ex);
		}
	};	
	ormmaview.createCalendarEvent = function(start, end, description, summary, location) {
		try {
			ORMMAUtilityControllerBridge.createCalendarEvent(start, end, description, summary, location);
		} catch (ex) {
			ormmaview.showAlert("createEvent: " + ex);
		}
	};
	ormmaview.makeCall = function(phoneNumber) {
		try {
			ORMMAUtilityControllerBridge.makeCall(phoneNumber);
		} catch (ex) {
			ormmaview.showAlert("makeCall: " + ex);
		}
	};
	ormmaview.sendMail = function(recipient, subject, body) {
		try {
			ORMMAUtilityControllerBridge.sendMail(recipient, subject, body);
		} catch (ex) {
			ormmaview.showAlert("sendMail: " + ex);
		}
	};
	ormmaview.sendSMS = function(recipient, body) {
		try {
			ORMMAUtilityControllerBridge.sendSMS(recipient, body);
		} catch (ex) {
			ormmaview.showAlert("sendSMS: " + ex);
		}
	};
	ormmaview.addAsset = function(URL, alias) {
		ORMMAAssetsControllerBridge.addAsset(URL, alias)  
	};
	ormmaview.getAssetURL = function(alias) {
		return ORMMAAssetsControllerBridge.getAssetUrl(alias)
	};
	ormmaview.removeAsset = function(alias) {
		//dummy
	};	
	ormmaview.setShakeProperties = function(properties) {
		//dummy
	};	
	ormmaview.getCacheRemaining = function() {
		return ORMMAAssetsControllerBridge.cacheRemaining()
	};
	ormmaview.request = function(URL, display) {
		ORMMANetworkControllerBridge.request(URL, display)
	};	
	ormmaview.getLocation = function() {
		ORMMALocationControllerBridge.getLocation();
	};
	ormmaview.getPlacementType = function() {
		return ORMMADisplayControllerBridge.getPlacementType();
	};
	ormmaview.stringify = function(args) {
		if (typeof JSON === "undefined") {
			var s = "";
			var len = args.length;
			var i;
			if (typeof len == "undefined") {
				return ormmaview.stringifyArg(args)
			}
			for (i = 0; i < args.length; i++) {
				if (i > 0) {
					s = s + ","
				}
				s = s + ormmaview.stringifyArg(args[i])
			}
			s = s + "]";
			return s
		} else {
			return JSON.stringify(args)
		}
	};
	ormmaview.stringifyArg = function(t) {
		var n, r, s, o, u, a;
		r = typeof t;
		n = "";
		if (r === "number" || r === "boolean") {
			n = n + args
		} else if (t instanceof Array) {
			n = n + "[" + t + "]"
		} else if (t instanceof Object) {
			s = true;
			n = n + "{";
			for (o in t) {
				if (t[o] !== null) {
					if (!s) {
						n = n + ","
					}
					n = n + '"' + o + '":';
					u = typeof t[o];
					if (u === "number" || u === "boolean") {
						n = n + t[o]
					} else if (typeof t[o] === "function") {
						n = n + '""'
					} else if (t[o] instanceof Object) {
						n = n + this.stringify(args[i][o])
					} else {
						n = n + '"' + t[o] + '"'
					}
					s = false
				}
			}
			n = n + "}"
		} else {
			a = t.replace(/\\\\/g, "\\\\\\\\");
			a = a.replace(/"/g, '\\\\"');
			n = n + '"' + a + '"'
		}
		ormmaview.showAlert("json:" + n);
		return n
	};
	var decode64 = function(e) {
		var t = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/=";
		var n = "";
		var r, i, s;
		var o, u, a, l;
		var c = 0;
		ormmaview = e.replace(/[^A-Za-z0-9\\+\\/\\=]/g, "");
		while (c < e.length) {
			o = t.indexOf(e.charAt(c++));
			u = t.indexOf(e.charAt(c++));
			a = t.indexOf(e.charAt(c++));
			l = t.indexOf(e.charAt(c++));
			r = o << 2 | u >> 4;
			i = (u & 15) << 4 | a >> 2;
			s = (a & 3) << 6 | l;
			n = n + String.fromCharCode(r);
			if (a != 64) {
				n = n + String.fromCharCode(i)
			}
			if (l != 64) {
				n = n + String.fromCharCode(s)
			}
		}
		n = f(n);
		return n
	};
	var f = function(e) {
		var t = "";
		var n = 0;
		var r = c1 = c2 = 0;
		while (n < e.length) {
			r = e.charCodeAt(n);
			if (r < 128) {
				t += String.fromCharCode(r);
				n++
			} else if (r > 191 && r < 224) {
				c2 = e.charCodeAt(n + 1);
				t += String.fromCharCode((r & 31) << 6 | c2 & 63);
				n += 2
			} else {
				c2 = e.charCodeAt(n + 1);
				c3 = e.charCodeAt(n + 2);
				t += String.fromCharCode((r & 15) << 12 | (c2 & 63) << 6 | c3 & 63);
				n += 3
			}
		}
		return t
	}
})();